#include "SceneEntity.h"
#include "Geometry.h"
#include "Material.h"
#include "RenderState.h"
#include "Shape.h"
#include "Transform3.h"
#include "Camera.h"

#include "glInterface.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>


using namespace std;


namespace CHCDemoEngine 
{


SceneEntity::SceneEntity(Transform3 *trafo): 
mTransform(trafo), mCurrentLODLevel(0)
{
	mBox.Initialize();
}


SceneEntity::~SceneEntity()
{
}


void SceneEntity::UpdateLODs(Camera *cam)
{
	const Vector3 pos = GetTransformedBoundingBox().Center();

	const float distance = SqrDistance(pos, cam->GetPosition());

	mCurrentLODLevel = 0;

	int l = (int)mLODLevels.size();

	for (int i = 0; i < l; ++ i)
	{
		LODLevel *lod = mLODLevels[i];

		if (lod->mSquaredDistance > distance)
			break;

		mCurrentLODLevel = i;
	}

	/*if (mLODLevels.size() > 1)
	{
		Debug << "dist: " << distance << " numshapes: " << mLODLevels[mCurrentLODLevel]->mShapes.size() << " lod: " << mCurrentLODLevel << " of " << mLODLevels.size() << endl;
	}*/
}

void SceneEntity::GetCurrentLODLevel(ShapeContainer::iterator &start, 
									 ShapeContainer::iterator &end)
{
	start = mLODLevels[mCurrentLODLevel]->mShapes.begin(); 
	end = mLODLevels[mCurrentLODLevel]->mShapes.end();
}


void SceneEntity::GetLODLevel(int level, 
							  ShapeContainer::iterator &start, 
							  ShapeContainer::iterator &end)
{
	LODLevel *lod = mLODLevels[level];

	start = lod->mShapes.begin();
	end = lod->mShapes.end();
}


void SceneEntity::Render(RenderState *state)
{
	ShapeContainer::iterator sit, sit_end;

	/*if (mShapes.size() > 8)
	{
		sit = mShapes.begin() + 6;
		sit_end = mShapes.begin() + 6;
	}
	else*/
		GetCurrentLODLevel(sit, sit_end);

	//if (mShapes.size() > 1)
		//mShapes[0]->Render(state);
	for (; sit != sit_end; ++ sit)
	{
		(*sit)->Render(state);
	}
}


void SceneEntity::AddShape(Shape *shape)
{
	mShapes.push_back(shape);
	mBox.Include(shape->GetBoundingBox());
}


void SceneEntity::SetTransform(Transform3 *trafo)
{
	mTransform = trafo;
}


void SceneEntity::SetLastRendered(int lastRendered)
{
	mLastRendered = lastRendered;
}


int SceneEntity::GetLastRendered() const
{
	return mLastRendered;
}


int SceneEntity::CountNumTriangles(int lodLevel)
{
	int numTriangles = 0;
	
	ShapeContainer::iterator sit, sit_end;

	if (lodLevel == -1)
	{
		GetCurrentLODLevel(sit, sit_end);
	}
	else
	{
		GetLODLevel(lodLevel, sit, sit_end);
	}


	for (; sit != sit_end; ++ sit)
	{
		numTriangles += (*sit)->GetGeometry()->GetNumTriangles();
	}

	return numTriangles;
}


AxisAlignedBox3 SceneEntity::GetTransformedBoundingBox() const
{
	Matrix4x4 *mat = mTransform->GetMatrix();

	if (!mat)
		return mBox;

	return Transform(mBox, *mat);
}


AxisAlignedBox3 SceneEntity::GetBoundingBox() const
{
	return mBox;
}

}